@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2023 Red Hat, Inc.
+// Copyright (C) 2013-2024 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -225,7 +225,7 @@ typedef unordered_map<string, method_decl_sptr> string_member_function_sptr_map;
 
 /// Convenience typedef for a map which key is a string and which
 /// value is a point to @ref var_decl.
-typedef unordered_map<string, const var_decl*> string_var_ptr_map;
+typedef unordered_map<string, var_decl_sptr> string_var_ptr_map;
 
 /// Convenience typedef for a pair of pointer to @ref var_decl
 /// representing a @ref var_decl change.  The first member of the pair
@@ -382,61 +382,69 @@ enum diff_category
   /// incompatible change to a vtable.
   VIRTUAL_MEMBER_CHANGE_CATEGORY = 1 << 12,
 
+  REFERENCE_LVALUENESS_CHANGE_CATEGORY = 1 << 13,
+
+  /// A change between two non-compatible types of different kinds.
+  NON_COMPATIBLE_DISTINCT_CHANGE_CATEGORY = 1 << 14,
+
+  /// A non-compatible name change between two types.
+  NON_COMPATIBLE_NAME_CHANGE_CATEGORY = 1 << 15,
+
   /// A diff node in this category is redundant.  That means it's
   /// present as a child of a other nodes in the diff tree.
-  REDUNDANT_CATEGORY = 1 << 13,
+  REDUNDANT_CATEGORY = 1 << 16,
 
   /// This means that a diff node in the sub-tree carries a type that
   /// was declaration-only and that is now defined, or vice versa.
-  TYPE_DECL_ONLY_DEF_CHANGE_CATEGORY = 1 << 14,
+  TYPE_DECL_ONLY_DEF_CHANGE_CATEGORY = 1 << 17,
 
   /// A diff node in this category is a function parameter type which
   /// top cv-qualifiers change.
-  FN_PARM_TYPE_TOP_CV_CHANGE_CATEGORY = 1 << 15,
+  FN_PARM_TYPE_TOP_CV_CHANGE_CATEGORY = 1 << 18,
 
   /// A diff node in this category has a function parameter type with a
   /// cv-qualifiers change.
-  FN_PARM_TYPE_CV_CHANGE_CATEGORY = 1 << 16,
+  FN_PARM_TYPE_CV_CHANGE_CATEGORY = 1 << 19,
 
   /// A diff node in this category is a function return type with a
   /// cv-qualifier change.
-  FN_RETURN_TYPE_CV_CHANGE_CATEGORY = 1 << 17,
+  FN_RETURN_TYPE_CV_CHANGE_CATEGORY = 1 << 20,
 
   /// A diff node in this category is a function (or function type)
   /// with at least one parameter added or removed.
-  FN_PARM_ADD_REMOVE_CHANGE_CATEGORY = 1 << 18,
+  FN_PARM_ADD_REMOVE_CHANGE_CATEGORY = 1 << 21,
 
   /// A diff node in this category is for a variable which type holds
   /// a cv-qualifier change.
-  VAR_TYPE_CV_CHANGE_CATEGORY = 1 << 19,
+  VAR_TYPE_CV_CHANGE_CATEGORY = 1 << 22,
 
   /// A diff node in this category carries a change from void pointer
   /// to non-void pointer.
-  VOID_PTR_TO_PTR_CHANGE_CATEGORY = 1 << 20,
+  VOID_PTR_TO_PTR_CHANGE_CATEGORY = 1 << 23,
 
   /// A diff node in this category carries a change in the size of the
   /// array type of a global variable, but the ELF size of the
   /// variable didn't change.
-  BENIGN_INFINITE_ARRAY_CHANGE_CATEGORY = 1 << 21,
+  BENIGN_INFINITE_ARRAY_CHANGE_CATEGORY = 1 << 24,
 
   /// A diff node in this category carries a change that must be
   /// reported, even if the diff node is also in the
   /// SUPPRESSED_CATEGORY or PRIVATE_TYPE_CATEGORY categories.
   /// Typically, this node matches a suppression specification like
   /// the [allow_type] directive.
-  HAS_ALLOWED_CHANGE_CATEGORY = 1 << 22,
+  HAS_ALLOWED_CHANGE_CATEGORY = 1 << 25,
 
   /// A diff node in this category has a descendant node that is in
   /// the HAS_ALLOWED_CHANGE_CATEGORY category.  Nodes in this
   /// category must be reported, even if they are also in the
   /// SUPPRESSED_CATEGORY or PRIVATE_TYPE_CATEGORY categories.
-  HAS_DESCENDANT_WITH_ALLOWED_CHANGE_CATEGORY = 1 << 23,
+  HAS_DESCENDANT_WITH_ALLOWED_CHANGE_CATEGORY = 1 << 26,
 
   /// A diff node in this category has a parent node that is in the
   /// HAS_ALLOWED_CHANGE_CATEGORY category.  Nodes in this category
   /// must be reported, even if they are also in the
   /// SUPPRESSED_CATEGORY or PRIVATE_TYPE_CATEGORY categories.
-  HAS_PARENT_WITH_ALLOWED_CHANGE_CATEGORY = 1 << 24,
+  HAS_PARENT_WITH_ALLOWED_CHANGE_CATEGORY = 1 << 26,
 
   /// A special enumerator that is the logical 'or' all the
   /// enumerators above.
@@ -457,6 +465,9 @@ enum diff_category
   | PRIVATE_TYPE_CATEGORY
   | SIZE_OR_OFFSET_CHANGE_CATEGORY
   | VIRTUAL_MEMBER_CHANGE_CATEGORY
+  | REFERENCE_LVALUENESS_CHANGE_CATEGORY
+  | NON_COMPATIBLE_DISTINCT_CHANGE_CATEGORY
+  | NON_COMPATIBLE_NAME_CHANGE_CATEGORY
   | REDUNDANT_CATEGORY
   | TYPE_DECL_ONLY_DEF_CHANGE_CATEGORY
   | FN_PARM_TYPE_TOP_CV_CHANGE_CATEGORY
@@ -1541,10 +1552,11 @@ class array_diff : public type_diff_base
   std::unique_ptr<priv> priv_;
 
 protected:
-  array_diff(const array_type_def_sptr	first,
-	     const array_type_def_sptr	second,
-	     diff_sptr			element_type_diff,
-	     diff_context_sptr		ctxt = diff_context_sptr());
+  array_diff(const array_type_def_sptr		first,
+	     const array_type_def_sptr		second,
+	     diff_sptr				element_type_diff,
+	     vector<subrange_diff_sptr>&	subrange_diffs,
+	     diff_context_sptr			ctxt = diff_context_sptr());
 
 public:
   const array_type_def_sptr
@@ -1556,9 +1568,18 @@ public:
   const diff_sptr&
   element_type_diff() const;
 
+  const vector<subrange_diff_sptr>&
+  subrange_diffs() const;
+
   void
   element_type_diff(diff_sptr);
 
+  void
+  subrange_diffs(const vector<subrange_diff_sptr>&);
+
+  bool
+  any_subrange_diff_to_be_reported() const;
+
   virtual const string&
   get_pretty_representation() const;
 
@@ -2980,6 +3001,9 @@ is_anonymous_class_or_union_diff(const diff* d);
 const subrange_diff*
 is_subrange_diff(const diff* diff);
 
+const subrange_diff*
+is_anonymous_subrange_diff(const diff* d);
+
 const array_diff*
 is_array_diff(const diff* diff);
 
