@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- mode: C++ -*-
 //
-// Copyright (C) 2013-2023 Red Hat, Inc.
+// Copyright (C) 2013-2024 Red Hat, Inc.
 
 /// @file
 
@@ -34,10 +34,10 @@ public:
   typedef std::unordered_set<const function_decl*> functions_set;
 
   ///Convenience typedef for std::vector<abigail::ir::var_decl*>
-  typedef vector<const var_decl*> variables;
+  typedef vector<var_decl_sptr> variables;
 
   /// Convenience typedef for std::unordered_set<const var_decl*>.
-  typedef std::unordered_set<const var_decl*> variables_set;
+  typedef std::unordered_set<var_decl_sptr> variables_set;
 
   class exported_decls_builder;
 
@@ -230,8 +230,11 @@ public:
   const std::unordered_set<function_decl*>*
   lookup_functions(const char* id) const;
 
-  const var_decl*
-  lookup_variable(const interned_string& id) const;
+  const std::unordered_set<var_decl_sptr>*
+  lookup_variables(const interned_string& id) const;
+
+  const std::unordered_set<var_decl_sptr>*
+  lookup_variables(const char* id) const;
 
   void
   sort_functions();
@@ -370,7 +373,7 @@ public:
   maybe_add_fn_to_exported_fns(function_decl*);
 
   bool
-  maybe_add_var_to_exported_vars(const var_decl*);
+  maybe_add_var_to_exported_vars(const var_decl_sptr&);
 }; //corpus::exported_decls_builder
 
 /// Abstraction of a group of corpora.
@@ -444,6 +447,8 @@ public:
   operator==(const corpus_group&) const;
 }; // end class corpus_group
 
+corpus_group_sptr
+is_corpus_group(const corpus_sptr&);
 }// end namespace ir
 }//end namespace abigail
 #endif //__ABG_CORPUS_H__
