@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2023 Red Hat, Inc.
+// Copyright (C) 2013-2024 Red Hat, Inc.
 
 /// @file
 
@@ -22,7 +22,6 @@
 #include <utility> // for std::rel_ops, at least.
 #include <vector>
 #include "abg-interned-str.h"
-#include "abg-hash.h"
 
 /// Toplevel namespace for libabigail.
 namespace abigail
@@ -359,13 +358,13 @@ const global_scope*
 get_global_scope(const decl_base_sptr);
 
 translation_unit*
-get_translation_unit(const decl_base&);
+get_translation_unit(const type_or_decl_base&);
 
 translation_unit*
-get_translation_unit(const decl_base*);
+get_translation_unit(const type_or_decl_base*);
 
 translation_unit*
-get_translation_unit(const decl_base_sptr);
+get_translation_unit(const type_or_decl_base_sptr&);
 
 bool
 is_global_scope(const scope_decl&);
@@ -451,6 +450,12 @@ is_integral_type(const type_or_decl_base*);
 type_decl_sptr
 is_integral_type(const type_or_decl_base_sptr&);
 
+type_decl*
+is_real_type(const type_or_decl_base*);
+
+type_decl_sptr
+is_real_type(const type_or_decl_base_sptr&);
+
 typedef_decl_sptr
 is_typedef(const type_or_decl_base_sptr);
 
@@ -655,6 +660,12 @@ look_through_decl_only(decl_base*);
 decl_base_sptr
 look_through_decl_only(const decl_base_sptr&);
 
+type_base*
+look_through_decl_only_type(type_base*);
+
+type_base_sptr
+look_through_decl_only_type(const type_base_sptr&);
+
 var_decl*
 is_var_decl(const type_or_decl_base*);
 
@@ -702,8 +713,8 @@ is_member_decl(const decl_base*);
 bool
 is_member_decl(const decl_base&);
 
-scope_decl*
-is_scope_decl(decl_base*);
+const scope_decl*
+is_scope_decl(const decl_base*);
 
 scope_decl_sptr
 is_scope_decl(const decl_base_sptr&);
@@ -950,14 +961,6 @@ get_member_function_vtable_offset(const function_decl&);
 ssize_t
 get_member_function_vtable_offset(const function_decl_sptr&);
 
-void
-set_member_function_vtable_offset(const function_decl& f,
-				  ssize_t s);
-
-void
-set_member_function_vtable_offset(const function_decl_sptr &f,
-				  ssize_t s);
-
 bool
 get_member_function_is_virtual(const function_decl&);
 
@@ -968,10 +971,11 @@ bool
 get_member_function_is_virtual(const function_decl*);
 
 void
-set_member_function_is_virtual(function_decl&, bool);
-
+set_member_function_virtuality(function_decl&, bool, ssize_t);
 void
-set_member_function_is_virtual(const function_decl_sptr&, bool);
+set_member_function_virtuality(function_decl*, bool, ssize_t);
+void
+set_member_function_virtuality(const function_decl_sptr&, bool, ssize_t);
 
 type_base_sptr
 strip_typedef(const type_base_sptr);
@@ -1572,7 +1576,7 @@ type_base_sptr
 type_or_void(const type_base_sptr, const environment&);
 
 type_base_sptr
-canonicalize(type_base_sptr);
+canonicalize(type_base_sptr type, bool do_log= false, bool show_stats= false);
 
 type_base*
 type_has_non_canonicalized_subtype(type_base_sptr t);
