@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2022-2023 Red Hat, Inc.
+// Copyright (C) 2022-2024 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -107,8 +107,8 @@ ir::corpus_sptr
 elf_based_reader::read_and_add_corpus_to_group(ir::corpus_group& group,
 					       fe_iface::status& status)
 {
-  group.add_corpus(corpus());
   ir::corpus_sptr corp = read_corpus(status);
+  group.add_corpus(corp);
   return corp;
 }
 
