@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2020-2023 Google, Inc.
+// Copyright (C) 2020-2024 Google, Inc.
 
 /// @file
 ///
@@ -287,6 +287,11 @@ e_machine_to_string(GElf_Half e_machine)
       return "elf-tilera-tilegx";
 #endif
 
+#ifdef HAVE_EM_RISCV_MACRO
+    case EM_RISCV:
+      return "elf-riscv";
+#endif
+
     case EM_NUM:
       return "elf-last-arch-number";
     case EM_ALPHA:
