@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2017-2023 Red Hat, Inc.
+// Copyright (C) 2017-2024 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -674,15 +674,21 @@ leaf_reporter::report(const class_or_union_diff& d,
 	}
 
       // report changes
-      size_t numchanges = (d.sorted_changed_data_members().size()
-			   + d.sorted_subtype_changed_data_members().size());
+      size_t net_numchanges = 0;
 
-      size_t num_filtered =
-	(d.count_filtered_changed_data_members(/*local_only=*/true)
-	 + d.count_filtered_subtype_changed_data_members(/*local_only=*/true));
+      for (var_diff_sptrs_type::const_iterator it =
+	     d.sorted_changed_data_members().begin();
+	   it != d.sorted_changed_data_members().end();
+	   ++it)
+	if (diff_to_be_reported((*it).get()))
+	  net_numchanges++;
 
-      ABG_ASSERT(numchanges >= num_filtered);
-      size_t net_numchanges = numchanges - num_filtered;
+      for (var_diff_sptrs_type::const_iterator it =
+	     d.sorted_subtype_changed_data_members().begin();
+	   it != d.sorted_subtype_changed_data_members().end();
+	   ++it)
+	if (diff_to_be_reported((*it).get()))
+	  net_numchanges++;
 
       if (net_numchanges)
 	{
@@ -1308,10 +1314,8 @@ leaf_reporter::report(const corpus_diff& d,
 	out << indent << num_changed
 	    << " Changed variables:\n\n";
       string n1, n2;
-      for (auto d : d.priv_->sorted_changed_vars_)
+      for (diff_sptr diff : d.priv_->sorted_changed_vars_)
 	{
-	  diff_sptr diff = d;
-
 	  if (!diff)
 	    continue;
 
