@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2023 Red Hat, Inc.
+// Copyright (C) 2013-2024 Red Hat, Inc.
 
 ///@file
 
@@ -416,9 +416,12 @@ is_regular_file(const string& path)
   if (S_ISREG(st.st_mode))
     return true;
 
-  string symlink_target_path;
-  if (maybe_get_symlink_target_file_path(path, symlink_target_path))
-    return is_regular_file(symlink_target_path);
+  if (S_ISLNK(st.st_mode))
+    {
+      string symlink_target_path;
+      if (maybe_get_symlink_target_file_path(path, symlink_target_path))
+	return is_regular_file(symlink_target_path);
+    }
 
   return false;
 }
@@ -2855,7 +2858,7 @@ get_binary_paths_from_kernel_dist(const string&	dist_root,
   // under the 'debug_info_root_path' directory and its content is
   // accessible from <debug_info_root_path>/usr/lib/debug directory.
 
-  string kernel_modules_root;
+  string kernel_modules_root = dist_root;
   string debug_info_root;
   if (dir_exists(dist_root + "/lib/modules"))
     {
@@ -3025,7 +3028,7 @@ load_vmlinux_corpus(elf_based_reader_sptr rdr,
 
   if (verbose)
     std::cerr << vmlinux
-     << " reading DONE:"
+     << " reading DONE in:"
      << t << "\n";
 
   if (group->is_empty())
@@ -3043,7 +3046,7 @@ load_vmlinux_corpus(elf_based_reader_sptr rdr,
          << *m << "' ("
          << cur_module_index
          << "/" << total_nb_modules
-         << ") ... " << std::flush;
+         << ") ...\n" << std::flush;
 
       rdr->initialize(*m, di_roots,
                       /*read_all_types=*/false,
@@ -3058,10 +3061,18 @@ load_vmlinux_corpus(elf_based_reader_sptr rdr,
       rdr->read_and_add_corpus_to_group(*group, status);
       t.stop();
       if (verbose)
-        std::cerr << "module '"
-         << *m
-         << "' reading DONE: "
-         << t << "\n";
+	std::cerr << "Module reading DONE in: "
+		  << t << " for '" << *m
+		  << "' (" << cur_module_index << "/" << total_nb_modules << ")"
+		  << "'\n";
+    }
+
+  if (verbose)
+    {
+      std::cerr << "Total number of functions: "
+		<< group->get_functions().size() << "\n";
+      std::cerr << "Total number of variables: "
+		<< group->get_variables().size() << "\n";
     }
 }
 
@@ -3122,7 +3133,7 @@ build_corpus_group_from_kernel_dist_under(const string&	root,
 	      << root
 	      << "' with vmlinux path: '"
 	      << vmlinux_path
-	      << "' ... " << std::flush;
+	      << "' ... \n" << std::flush;
 
   timer t;
 
@@ -3132,7 +3143,7 @@ build_corpus_group_from_kernel_dist_under(const string&	root,
   t.stop();
 
   if (verbose)
-    std::cerr << "DONE: " << t << "\n";
+    std::cerr << "Kernel tree binary paths analysis DONE in: " << t << "\n";
 
   if (got_binary_paths)
     {
@@ -3144,10 +3155,11 @@ build_corpus_group_from_kernel_dist_under(const string&	root,
 
 #ifdef WITH_CTF
       shared_ptr<char> di_root_ctf;
+      char *di_root_ctf_ptr;
       if (requested_fe_kind & corpus::CTF_ORIGIN)
         {
           di_root_ctf = make_path_absolute(root.c_str());
-          char *di_root_ctf_ptr = di_root_ctf.get();
+          di_root_ctf_ptr = di_root_ctf.get();
           di_roots.push_back(&di_root_ctf_ptr);
         }
 #endif
@@ -3227,7 +3239,8 @@ create_best_elf_based_reader(const string& elf_file_path,
     {
 #ifdef WITH_BTF
       if (file_has_btf_debug_info(elf_file_path, debug_info_root_paths))
-	result = btf::create_reader(elf_file_path, debug_info_root_paths, env);
+	result = btf::create_reader(elf_file_path, debug_info_root_paths, env,
+				    show_all_types, linux_kernel_mode);
 #endif
     }
   else
@@ -3246,7 +3259,8 @@ create_best_elf_based_reader(const string& elf_file_path,
 	  && file_has_btf_debug_info(elf_file_path, debug_info_root_paths))
 	// The file has BTF debug info and no BTF, let's use the BTF
 	// front-end even if it wasn't formally requested by the user.
-	result = btf::create_reader(elf_file_path, debug_info_root_paths, env);
+	result = btf::create_reader(elf_file_path, debug_info_root_paths, env,
+				    show_all_types, linux_kernel_mode);
 #endif
     }
 
