@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2023 Red Hat, Inc.
+// Copyright (C) 2013-2024 Red Hat, Inc.
 //
 // Author: Sinny Kumari
 
@@ -772,6 +772,31 @@ static InOutSpec in_out_specs[] =
     "data/test-diff-pkg/PR29610/guestfs-tools-1.51.6-2.el9.s390x-self-check-report.txt",
     "output/test-diff-pkg/PR29610/guestfs-tools-1.51.6-2.el9.s390x-self-check-report.txt"
   },
+  {
+    "data/test-diff-pkg/libgm2-14.2.1-1.fc40.x86_64.rpm",
+    "data/test-diff-pkg/libgm2-14.2.1-1.fc40.x86_64.rpm",
+    "--self-check",
+    "",
+    "data/test-diff-pkg/libgm2-debuginfo-14.2.1-1.fc40.x86_64.rpm, "
+    "data/test-diff-pkg/gcc-debuginfo-14.2.1-1.fc40.x86_64.rpm",
+    "data/test-diff-pkg/libgm2-debuginfo-14.2.1-1.fc40.x86_64.rpm",
+    "",
+    "",
+    "data/test-diff-pkg/libgm2-14.2.1-1.fc40.x86_64-self-check-report.txt",
+    "output/test-diff-pkg/libgm2-14.2.1-1.fc40.x86_64-self-check-report.txt"
+  },
+  {
+    "data/test-diff-pkg/infinipath-psm-3.3-26_g604758e_open.6.fc36.5.x86_64.rpm",
+    "data/test-diff-pkg/infinipath-psm-3.3-26_g604758e_open.6.fc36.5.x86_64.rpm",
+    "--self-check",
+    "",
+    "data/test-diff-pkg/infinipath-psm-debuginfo-3.3-26_g604758e_open.6.fc36.5.x86_64.rpm",
+    "data/test-diff-pkg/infinipath-psm-debuginfo-3.3-26_g604758e_open.6.fc36.5.x86_64.rpm",
+    "",
+    "",
+    "data/test-diff-pkg/infinipath-psm-3.3-26_g604758e_open.6.fc36.5.x86_64-self-check-report.txt",
+    "output/test-diff-pkg/infinipath-psm-3.3-26_g604758e_open.6.fc36.5.x86_64-self-check-report.txt"
+  },
 #endif // WITH_RPM_ZSTD
 #endif //WITH_RPM
 
