@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2014-2023 Red Hat, Inc.
+// Copyright (C) 2014-2024 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -166,11 +166,11 @@ struct fn_change
 /// the differences found in the type of that variable.
 struct var_change
 {
-  const var_decl* decl = nullptr;
+  var_decl_sptr decl = nullptr;
   diff_sptr diff;
   bool reverse_direction = false;
 
-  var_change(const var_decl* var,
+  var_change(const var_decl_sptr& var,
 	     diff_sptr difference,
 	     bool reverse_dir)
     : decl(var),
@@ -542,27 +542,30 @@ compare_expected_against_provided_variables(diff_context_sptr&		ctxt,
     {
       interned_string var_id = expected_var->get_id();
       // ... against the variables exported by the library!
-      const var_decl* exported_var =
+      const std::unordered_set<var_decl_sptr>* exported_vars =
 	reverse_direction
-	? app_corpus->lookup_variable(var_id)
-	: lib_corpus->lookup_variable(var_id);
-      if (exported_var)
+	? app_corpus->lookup_variables(var_id)
+	: lib_corpus->lookup_variables(var_id);
+      if (exported_vars)
 	{
-	  // OK here is where we compare the variable expected by
-	  // the application against the variable exported by the
-	  // library.
-	  diff_sptr type_diff =
-	    compute_diff(expected_var->get_type(),
-			 exported_var->get_type(),
-			 ctxt);
-	  if (type_diff && type_diff->to_be_reported())
+	  for (auto exported_var : *exported_vars)
 	    {
-	      // So there is a type change between the variable
-	      // expected by the application and the variable
-	      // exported by the library.  Let's record that
-	      // change so that we can report about it later.
-	      var_changes.push_back(var_change(expected_var, type_diff, reverse_direction));
-	      status |= abigail::tools_utils::ABIDIFF_ABI_CHANGE;
+	      // OK here is where we compare the variable expected by
+	      // the application against the variable exported by the
+	      // library.
+	      diff_sptr type_diff =
+		compute_diff(expected_var->get_type(),
+			     exported_var->get_type(),
+			     ctxt);
+	      if (type_diff && type_diff->to_be_reported())
+		{
+		  // So there is a type change between the variable
+		  // expected by the application and the variable
+		  // exported by the library.  Let's record that
+		  // change so that we can report about it later.
+		  var_changes.push_back(var_change(expected_var, type_diff, reverse_direction));
+		  status |= abigail::tools_utils::ABIDIFF_ABI_CHANGE;
+		}
 	    }
 	}
     }
